!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: bomlast.gs,v 1.7 2008-01-09 19:02:29 stever Exp $
!
! Description - 
!  Final changes to kernel classes and reserved objects during filein.
!  Repository conversion of AllGroups, Segments and UserProfiles
!=========================================================================

!input all the changes that are used only during upgrades.
input $upgradeDir/bomlastconv.gs

!------------------------------------------------------------
! Build the table of global shared dependency lists
!------------------------------------------------------------
run
| seg depListTable |
" create a new segment that is world writable "
seg := Segment newInRepository: SystemRepository.
seg worldAuthorization: #write.

" don't allow anyone else to change the authorization of the new segment "
seg changeToSegment: SystemSegment.
seg immediateInvariant.
Globals at: #GsIndexingSegment put: seg.

depListTable := Globals at: #SharedDependencyLists otherwise: nil.
depListTable == nil
  ifTrue: [
    "We are creating a new GemStone repository."
    Globals at: #SharedDependencyLists put: DepListTable new
    ]
  ifFalse: [
    "We are in conversion. Just reinitialize the existing DepListTable"
    depListTable reinitialize.
    ].

^ true
%

!------------------------------------------------------------
!  rename the all users collections
run
Globals at:#OldAllUsers put: (Globals at:#AllUsers) .
Globals at:#AllUsers put: (Globals at:#NewAllUsers) .
Globals removeKey: #NewAllUsers .
^ true
%

!------------------------------------------------------------
! dereference the hash dictionary on old AllUsers 
!------------------------------------------------------------
expectvalue true
run
OldAllUsers tagAt: 1 put: nil .
^ true
%

!------------------------------------------------------------
! create security data segment owned by SystemUser
run
| newSeg segNumber |
newSeg := Segment newInRepository: SystemRepository .
segNumber := newSeg number .
newSeg := newSeg _unsafeSetOop: 3685 .
Globals at:#SecurityDataSegment put: newSeg .
SystemRepository basicAt: segNumber put: newSeg .
^ true
%

!------------------------------------------------------------
! create security data dictionary and userId dictionary on AllUsers
run
AllUsers _initialize  .
^ true
%

!------------------------------------------------------------
! Migrate users from old to new set
run
| saveArray saveSeg |
saveSeg := System currentSegment .
saveArray := Array new .
OldAllUsers do:[ :aUserProfile |
  AllUsers add: aUserProfile .
  saveArray add: aUserProfile .
  ].

saveArray do:[:aUserProfile |
  OldAllUsers remove: aUserProfile .
  ].

saveArray size: 0 .
^ true
%

! build dictionary on new AllUsers - deleted

commit
!------------------------------------------------------------
! Add the Published dictionary to all UserProfiles' symbol lists
!  Add new groups to AllGroups
!------------------------------------------------------------
run
| newDict published publishedSeg saveSegment |
saveSegment := System currentSegment.
System currentSegment: DataCuratorSegment .

"convert AllGroups to a dictionary."
AllGroups class == CanonicalStringDictionary ifFalse:[
  newDict := CanonicalStringDictionary new .
  AllGroups do:[ :aSym | | aStr | 
    aStr := (String withAll: aSym ) immediateInvariant .
    newDict add: aStr .
    ].
  AllGroups := newDict .
  ].
AllGroups add: 'System' ; add: 'Publishers'; add:'Subscribers' .

publishedSeg := Segment newInRepository: SystemRepository .
publishedSeg group: 'Subscribers' authorization: #read ;
             group: 'Publishers'  authorization: #write .

System currentSegment: publishedSeg.

published := SymbolDictionary new name: #Published ; yourself . 

System currentSegment: DataCuratorSegment .

Globals at:#PublishedSegment put: publishedSeg .

SystemRepository _migrateGroups .

AllUsers do:[:aUserPro | 
  aUserPro _migrateGroups .
  aUserPro symbolList addLast: published .
  aUserPro addGroup: 'Subscribers' .
  ].

(AllUsers userWithId: 'DataCurator') 
   addGroup: 'System' ;
   addGroup: 'Publishers' .

(AllUsers userWithId: 'SystemUser') 
   addGroup: 'System' ;
   addGroup: 'Publishers' .

System currentSegment: saveSegment. 
^ true
%

!------------------------------------------------------------
! Populate the entry in Globals for collecting Garbage Collection Candidates.
! Create it in GsIndexingSegment to avoid authorization errors.
!------------------------------------------------------------
run
| saveSegment |

saveSegment := System currentSegment.
System currentSegment: GsIndexingSegment .
Globals at: #GcCandidates put: RcQueue new.
Globals at: #GcCandidatesCount put: RcCounter new.
Globals at: #GcWeakReferences put: Array new.
System currentSegment: saveSegment.
^true
%
!------------------------------------------------------------
! Add the SharedDependencyLists so that they are treated as weak references 
!------------------------------------------------------------
run
GcWeakReferences add: SharedDependencyLists.
^true
%
!-------------
! Add new Global as part of fix for bug 19391
expectvalue false
run
(Globals at: #AddSystemObjectsToGcCandidates otherwise: nil) == nil
  ifTrue:[Globals at: #AddSystemObjectsToGcCandidates put: false].
^false.
%

!-------------
! removal of _unsafesetoop method done later in filein
 
! end of bomlast.gs
commit
